"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = __importStar(require("react"));
var react_dom_1 = __importStar(require("react-dom"));
var getDisplayName_1 = __importDefault(require("./getDisplayName"));
function checkMountModeEnabled() {
    // @ts-ignore
    if (Cypress.spec.specType !== 'component') {
        throw new Error("In order to use mount or unmount functions please place the spec in component folder");
    }
}
/**
 * Inject custom style text or CSS file or 3rd party style resources
 */
var injectStyles = function (options) { return function () {
    var document = cy.state('document');
    var el = document.getElementById('cypress-jsdom');
    // insert any custom global styles before the component
    if (typeof options.stylesheets === 'string') {
        options.stylesheets = [options.stylesheets];
    }
    if (Array.isArray(options.stylesheets)) {
        // console.log('adding stylesheets')
        options.stylesheets.forEach(function (href) {
            var link = document.createElement('link');
            link.type = 'text/css';
            link.rel = 'stylesheet';
            link.href = href;
            document.body.insertBefore(link, el);
        });
    }
    if (options.style) {
        var style = document.createElement('style');
        style.appendChild(document.createTextNode(options.style));
        document.body.insertBefore(style, el);
    }
    if (options.cssFile) {
        return cy.readFile(options.cssFile).then(function (css) {
            var style = document.createElement('style');
            style.appendChild(document.createTextNode(css));
            document.body.appendChild(style);
        });
    }
}; };
/**
 * Mount a React component in a blank document; register it as an alias
 * To access: use an alias or original component reference
 *  @function   cy.mount
 *  @param      {React.ReactElement}  jsx - component to mount
 *  @param      {string}  [Component] - alias to use later
 *  @example
 ```
 import Hello from './hello.jsx'
 // mount and access by alias
 cy.mount(<Hello />, 'Hello')
 // using default alias
 cy.get('@Component')
 // using original component
 cy.get(Hello)
 ```
 **/
exports.mount = function (jsx, options) {
    if (options === void 0) { options = {}; }
    checkMountModeEnabled();
    // Get the display name property via the component constructor
    // @ts-ignore FIXME
    var displayname = getDisplayName_1.default(jsx.type, options.alias);
    return cy
        .window({ log: false })
        .then(function () {
        Cypress.log({
            name: 'mount',
            message: ["ReactDOM.render(<" + displayname + " ... />)"],
            consoleProps: function () {
                return {
                    props: jsx.props,
                };
            },
        });
    })
        .then(injectStyles(options))
        .then(function () {
        var _a, _b, _c, _d;
        var document = cy.state('document');
        var reactDomToUse = options.ReactDom || react_dom_1.default;
        var el = document.getElementById('cypress-jsdom');
        var props = {
            // @ts-ignore provide unique key to the the wrapped component to make sure we are rerendering between tests
            key: ((_d = (_c = (_b = (_a = Cypress) === null || _a === void 0 ? void 0 : _a.mocha) === null || _b === void 0 ? void 0 : _b.getRunner()) === null || _c === void 0 ? void 0 : _c.test) === null || _d === void 0 ? void 0 : _d.title) || Math.random(),
        };
        var CypressTestComponent = reactDomToUse.render(React.createElement(React.Fragment, props, jsx), el);
        cy.wrap(CypressTestComponent, { log: false }).as(options.alias || displayname);
    });
};
/**
 * Removes any mounted component
 */
exports.unmount = function () {
    checkMountModeEnabled();
    cy.log('unmounting...');
    return cy.get('#cypress-jsdom', { log: false }).then(function ($el) {
        react_dom_1.unmountComponentAtNode($el[0]);
    });
};
exports.default = exports.mount;
